import type { Message } from '@tencentcloud/chat-react-native';
import type { IMessageHandler, ITUIChatService } from '../interface/service';
import { JSONToObject, formatTime, formateFileSize } from '../utils/common-utils';
import { BASEURL, BASIC_EMOJI_URL_MAPPING } from '../config';
import { StoreName } from '../const';

export default class MessageHandler implements IMessageHandler {
  private TUIChatService: ITUIChatService;
  private userShowNameMap: Map<string, string>; // Map<userID, showName>
  private requestedUserMap: Map<string, number>; // Map<userID, 1>
  constructor(TUIChatService: ITUIChatService) {
    this.TUIChatService = TUIChatService;
    this.userShowNameMap = new Map();
    this.requestedUserMap = new Map();
  }

  private getEngine() {
    return this.TUIChatService.getEngine();
  }

  /**
   * 国际化翻译
   * @param {string} text 待翻译的文本
   * @returns {string}
  */
  private t(text: string) {
    return this.getEngine().TUITranslate.t(text);
  }

  handleTextMessage(message: Message) {
    const ret: any = {
      text: this.decodeText(message.payload),
    };
    return ret;
  }

  handleFaceMessage(message: Message) {
    const ret: any = {
      name: '',
      url: '',
      type: '',
    };
    // 自定义 face 表情图片名称必须带 @custom 标识，比如： '@custom_test.png'
    if (message.payload.data.indexOf('@custom') > -1) {
      ret.name = message.payload.data;
      ret.type = 'custom';
      return ret;
    }
    ret.name = message.payload.data.indexOf('@2x') < 0 ? `${message.payload.data}@2x` : message.payload.data;
    ret.url = `${BASEURL.face}${ret.name}.png`;
    return ret;
  }

  handleLocationMessage(message: Message) {
    const ret: any = {
      lon: '',
      lat: '',
      href: '',
      url: '',
      description: '',
    };
    ret.lon = message.payload.longitude.toFixed(6);
    ret.lat = message.payload.latitude.toFixed(6);
    ret.href = `${BASEURL.mapHref}pointx=${ret.lon}&pointy=${ret.lat}&name=${message.payload.description}`;
    ret.url = `${BASEURL.mapUrl}center=${ret.lat},${ret.lon}&zoom=10&size=300*150&maptype=roadmap&markers=size:large|color:0xFFCCFF|label:k|${ret.lat},${ret.lon}&key=UBNBZ-PTP3P-TE7DB-LHRTI-Y4YLE-VWBBD`;
    ret.description = message.payload.description;
    return ret;
  }

  handleImageMessage(message: Message) {
    const cacheUrl = this.getEngine().TUIStore.getData(StoreName.CHAT, "cacheMap").get(message.ID);
    return {
      url: cacheUrl ?? message.payload.imageInfoArray[0].url,
      width: message.payload.imageInfoArray[0].width,
      height: message.payload.imageInfoArray[0].height,
    };
  }

  handleAudioMessage(message: Message) {
    const cacheUrl = this.getEngine().TUIStore.getData(StoreName.CHAT, "cacheMap").get(message.ID);
    return {
      url: cacheUrl ?? message.payload.url,
      second: message.payload.second,
    };
  }

  handleVideoMessage(message: Message) {
    const chatEngine = this.getEngine();
    const cacheMap = chatEngine.TUIStore.getData(StoreName.CHAT, "cacheMap");
    const cacheVideoUrl = cacheMap.get(message.ID);
    const cacheSnapShotUrl = cacheMap.get(message.ID + '_S');
    return {
      url: cacheVideoUrl ?? message.payload.videoUrl,
      snapshotUrl: cacheSnapShotUrl ?? message.payload.snapshotUrl,
      snapshotWidth: message.payload.snapshotWidth,
      snapshotHeight: message.payload.snapshotHeight,
    };
  }

  handleFileMessage(message: Message) {
    const cacheUrl = this.getEngine().TUIStore.getData(StoreName.CHAT, "cacheMap").get(message.ID);
    return {
      url: cacheUrl ?? message.payload.fileUrl,
      name: message.payload.fileName,
      size: formateFileSize(message.payload.fileSize),
    };
  }

  handleCustomMessage(message: Message) {
    const createGroupText = this.handleCreateGroupCustomMessage(message);
    return {
      custom: this.handleCallKitSignaling(message) || createGroupText || message?.payload?.extension || `${this.t('TUIChat.[自定义消息]')}`,
      businessID: createGroupText ? 'group_create' : '',
    };
  }

  handleMergeMessage(message: Message) {
    return { ...message.payload };
  }

  handleGroupTipsMessage(message: Message) {
    const chatEngine = this.getEngine();
    const ret: any = {
      text: '',
    };

    // 优先取 nick，nick 不存在时取 userID，以下逻辑是为兼容不同群提示消息的处理
    let showName: string = message?.nick || message?.payload?.userIDList?.join(',');
    if (message?.payload?.memberList?.length > 0) {
      showName = '';
      message?.payload?.memberList?.map((user: any) => {
        const _showName = user?.nick || user?.userID;
        showName += `${this.substringByLength(_showName)},`;
        return user;
      });
      showName = showName?.slice(0, -1);
    }

    switch (message.payload.operationType) {
      case chatEngine.TYPES.GRP_TIP_MBR_JOIN:
        ret.text = `${showName} ${this.t('message.tip.加入群组')}`;
        break;
      case chatEngine.TYPES.GRP_TIP_MBR_QUIT:
        ret.text = `${this.t('message.tip.群成员')}：${message.payload?.operatorInfo?.nickName ?? message.payload?.operatorInfo?.userID ?? "" } ${this.t('message.tip.退出群组')}`;
        break; 
      case chatEngine.TYPES.GRP_TIP_MBR_KICKED_OUT:
        ret.text = `${this.t('message.tip.群成员')}：${showName} ${this.t('message.tip.被')}${this.t('message.tip.踢出群组')}`;
        break;
      case chatEngine.TYPES.GRP_TIP_MBR_SET_ADMIN:
        ret.text = `${this.t('message.tip.群成员')}：${showName} ${this.t('message.tip.成为管理员')}`;
        break;
      case chatEngine.TYPES.GRP_TIP_MBR_CANCELED_ADMIN:
        ret.text = `${this.t('message.tip.群成员')}：${showName} ${this.t('message.tip.被撤销管理员')}`;
        break;
      case chatEngine.TYPES.GRP_TIP_GRP_PROFILE_UPDATED:
        ret.text = this.handleGroupProfileUpdated(message);
        break;
      case chatEngine.TYPES.GRP_TIP_MBR_PROFILE_UPDATED:
        for (const member of message.payload.memberList) {
          if (member.muteTime > 0) {
            ret.text = `${this.t('message.tip.群成员')}：${showName} ${this.t('message.tip.被禁言')}`;
          } else {
            ret.text = `${this.t('message.tip.群成员')}：${showName} ${this.t('message.tip.被取消禁言')}`;
          }
        }
        break;
      default:
        ret.text = `[${this.t('message.tip.群提示消息')}]`;
        break;
    }
    return ret;
  }

  handleGroupSystemMessage(message: Message) {
    const groupName = message.payload.groupProfile.name || message.payload.groupProfile.groupID;
    const ret = {
      text: '',
    };
    switch (message.payload.operationType) {
      case 1:
        ret.text = `${message.payload.operatorID} ${this.t('message.tip.申请加入群组')}：${groupName}`;
        break;
      case 2:
        ret.text = `${this.t('message.tip.成功加入群组')}：${groupName}`;
        break;
      case 3:
        ret.text = `${this.t('message.tip.申请加入群组')}：${groupName} ${this.t('message.tip.被拒绝')}`;
        break;
      case 4:
        ret.text = `${this.t('message.tip.你被管理员')}${message.payload.operatorID} ${this.t('message.tip.踢出群组')}：${groupName}`;
        break;
      case 5:
        ret.text = `${this.t('message.tip.群')}：${groupName} ${this.t('message.tip.被')} ${message.payload.operatorID} ${this.t('message.tip.解散')}`;
        break;
      case 6:
        ret.text = `${message.payload.operatorID} ${this.t('message.tip.创建群')}：${groupName}`;
        break;
      case 7:
        ret.text = `${message.payload.operatorID} ${this.t('message.tip.邀请你加群')}：${groupName}`;
        break;
      case 8:
        ret.text = `${this.t('message.tip.你退出群组')}：${groupName}`;
        break;
      case 9:
        ret.text = `${this.t('message.tip.你被')}${message.payload.operatorID} ${this.t('message.tip.设置为群')}：${groupName} ${this.t('message.tip.的管理员')}`;
        break;
      case 10:
        ret.text = `${this.t('message.tip.你被')}${message.payload.operatorID} ${this.t('message.tip.撤销群')}：${groupName} ${this.t('message.tip.的管理员身份')}`;
        break;
      case 12:
        ret.text = `${message.payload.operatorID} ${this.t('message.tip.邀请你加群')}：${groupName}`;
        break;
      case 13:
        ret.text = `${message.payload.operatorID} ${this.t('message.tip.同意加群')}：${groupName}`;
        break;
      case 14:
        ret.text = `${message.payload.operatorID} ${this.t('message.tip.拒接加群')}：${groupName}`;
        break;
      case 255:
        ret.text = `${this.t('message.tip.自定义群系统通知')}: ${message.payload.userDefinedField}`;
        break;
      default:
        ret.text = '未解析的群系统通知';
        break;
    }
    return ret;
  }

  handleCallKitSignaling(message: any) {
    const callMessage: any = JSONToObject(message.payload.data);
    if (callMessage?.businessID !== 1) {
      return '';
    }
    const objectData = JSONToObject(callMessage?.data);
    const userID = message.fromAccount || message.from; // 取 message.fromAccount 兼容会话 lastMsg
    const friendRemark: any = this.getEngine().TUIFriend.getFriendRemark([userID]);
    const myUserID = this.getEngine().getMyUserID();
    let messageSender = friendRemark[userID] || message.nameCard || message.nick || userID;
    messageSender = this.substringByLength(messageSender);
    switch (callMessage?.actionType) {
      case 1: {
        if (objectData?.data?.cmd === 'audioCall' || objectData?.data?.cmd === 'videoCall') {
          if (callMessage?.groupID) {
            return `${messageSender} ${this.t('message.custom.发起通话')}`;
          }
        }
        if (objectData?.data?.cmd === 'hangup') {
          if (callMessage?.groupID) {
            return `${this.t('message.custom.通话结束')}`;
          }
          return `${this.t('message.custom.通话时长')}：${formatTime(objectData?.call_end)}`;
        }
        if (objectData?.data?.cmd === 'switchToAudio') {
          return `${this.t('message.custom.切换语音通话')}`;
        }
        if (objectData?.data?.cmd === 'switchToVideo') {
          return `${this.t('message.custom.切换视频通话')}`;
        }
        // CDM 异常时【发起通话】作为默认返回值
        return `${this.t('message.custom.发起通话')}`;
      }
      case 2:
        if (callMessage?.groupID) {
          return `${messageSender} ${this.t('message.custom.取消通话')}`;
        }
        if (this.isOldUIKit('message.custom.已取消')) {
          return this.t('message.custom.取消通话');
        }
        if (callMessage?.inviter === myUserID) {
          return this.t('message.custom.已取消');
        }
        return this.t('message.custom.对方已取消');
      case 3:
        if (objectData?.data?.cmd === 'switchToAudio') {
          return `${this.t('message.custom.切换语音通话')}`;
        }
        if (objectData?.data?.cmd === 'switchToVideo') {
          return `${this.t('message.custom.切换视频通话')}`;
        }
        if (callMessage?.groupID) {
          return `${messageSender} ${this.t('message.custom.已接听')}`;
        }
        return this.t('message.custom.已接听');
      case 4:
        if (callMessage?.groupID) {
          return `${messageSender} ${this.t('message.custom.拒绝通话')}`;
        }
        if (this.isOldUIKit('message.custom.已拒绝')) {
          return this.t('message.custom.拒绝通话');
        }
        if (objectData?.line_busy === 'line_busy' || objectData?.data.message === 'lineBusy') {
          if (callMessage?.inviter === myUserID) {
            return this.t('message.custom.对方忙线中');
          }
          return this.t('message.custom.忙线未接听');
        }
        if (callMessage?.inviter === myUserID) {
          return this.t('message.custom.对方已拒绝');
        }
        return this.t('message.custom.已拒绝');
      case 5:
        if (objectData?.data?.cmd === 'switchToAudio') {
          return `${this.t('message.custom.切换语音通话')}`;
        }
        if (objectData?.data?.cmd === 'switchToVideo') {
          return `${this.t('message.custom.切换视频通话')}`;
        }
        if (callMessage?.groupID) {
          // 群通话主叫超时
          if (userID === callMessage?.inviter) {
            this.handleCallkitTimeoutSignaling(callMessage.inviteeList);
            let inviteeList = '';
            callMessage.inviteeList?.forEach((inviteeUserID: string) => {
              const showName = this.userShowNameMap.get(inviteeUserID) || inviteeUserID;
              inviteeList += `${this.substringByLength(showName)}、`;
            });
            inviteeList = inviteeList.substring(0, inviteeList.lastIndexOf('、'));
            return `${inviteeList} ${this.t('message.custom.无应答')}`;
          }
          // 群通话被叫超时
          return `${messageSender} ${this.t('message.custom.无应答')}`;
        }
        if (this.isOldUIKit('message.custom.对方无应答')) {
          return this.t('message.custom.无应答');
        }
        if (callMessage?.inviter === myUserID) {
          return this.t('message.custom.对方无应答');
        }
        return this.t('message.custom.超时无应答');
      default:
        return '';
    }
  }

  private handleCreateGroupCustomMessage(message: Message) {
    let text: undefined | string;
    const data = JSONToObject(message.payload.data);
    if (data?.businessID === 'group_create') {
      text = `${data.opUser} ${data.content}`;
    }
    return text;
  }

  /**
   * 解析文本消息
   * @param {Object} payload 消息 payload 载体
   * @param {String} payload.text 消息内容
   * @returns {String}
  */
  private decodeText(payload: { text: string }) {
    const renderDom = [];
    // 文本消息
    let temp = payload.text;
    let left = -1;
    let right = -1;
    while (temp !== '') {
      left = temp.indexOf('[');
      right = temp.indexOf(']');
      switch (left) {
        case 0:
          if (right === -1) {
            renderDom.push({
              name: 'text',
              text: temp,
            });
            temp = '';
          } else {
            const emojiKey = temp.slice(0, right + 1);
            // 自定义 emoji 表情 emojiKey 必须带 @custom 标识，比如：'[@custom_Smile]'
            if (emojiKey.indexOf('@custom') > -1) {
              renderDom.push({
                name: 'img',
                src: '',
                type: 'custom',
                emojiKey,
              });
              temp = temp.substring(right + 1);
            } else if (BASIC_EMOJI_URL_MAPPING[emojiKey]) {
              renderDom.push({
                name: 'img',
                src: BASEURL.emoji + BASIC_EMOJI_URL_MAPPING[emojiKey],
                emojiKey,
              });
              temp = temp.substring(right + 1);
            } else {
              renderDom.push({
                name: 'text',
                text: '[',
              });
              temp = temp.slice(1);
            }
          }
          break;
        case -1:
          renderDom.push({
            name: 'text',
            text: temp,
          });
          temp = '';
          break;
        default:
          renderDom.push({
            name: 'text',
            text: temp.slice(0, left),
          });
          temp = temp.substring(left);
          break;
      }
    }
    return renderDom;
  }

  /**
   * 解析群资料更新内容
   * @param {Message} message 群提示消息
   * @returns {String}
  */
  private handleGroupProfileUpdated(message:Message) {
    const { nick, payload } = message;
    const { newGroupProfile, memberList, operatorID } = payload;
    let text = '';
    const showName = nick || operatorID;
    const key = Object.keys(newGroupProfile)[0];
    switch (key) {
      case 'muteAllMembers':
        if (newGroupProfile[key]) {
          text = `${this.t('message.tip.管理员')} ${showName} ${this.t('message.tip.开启全员禁言')}`;
        }
        else {
          text = `${this.t('message.tip.管理员')} ${showName} ${this.t('message.tip.取消全员禁言')}`;
        }
        break;
      case 'groupCustomField':
        const childKey = newGroupProfile.groupCustomField[0].key;
        switch (childKey) {
                    case 'ownerID':
            text = `${memberList[0].nick || memberList[0].userID} ${this.t('message.tip.成为新的群主')}`;
            break;
          case 'name':
            text = `${showName} ${this.t('message.tip.修改群名为')} ${newGroupProfile.groupCustomField[0].value}`;
            break;
          case 'notification':
            if (this.isOldUIKit('message.tip.把群公告修改为')) {
              text = `${showName} ${this.t('message.tip.发布新公告')}`;
            }
            else {
              text = `${showName} ${this.t('message.tip.把群公告修改为')} "${newGroupProfile.groupCustomField[0].value}"`;
            }
            break;
          case 'joinOption':
            text = `${showName} ${'修改进群选项'}`;
            break;
          case 'inviteOption':
            text = `${showName} ${'修改邀请进群选项'}`;
            break;
        }
        break;
    }
    return text;
  }

  private handleCallkitTimeoutSignaling(inviteeList: string[] = []) {
    if (inviteeList.length === 0) {
      return;
    }
    const friendRemarkResult = this.getEngine().TUIFriend.getFriendRemark(inviteeList);
    const userIDList: string[] = [];
    inviteeList.forEach((userID: string) => {
      const showName = friendRemarkResult[userID];
      if (showName) {
        this.userShowNameMap.set(userID, showName);
      } else if (!this.requestedUserMap.has(userID)) {
        userIDList.push(userID);
        this.requestedUserMap.set(userID, 1);
      }
    });
    // 注意：按照一条主叫超时信令请求一次处理，先不用截流处理
    if (userIDList.length > 0) {
      this.getEngine().TUIUser.getUserProfile({ userIDList }).then((imResponse: any) => {
        const profileList = imResponse.data || [];
        profileList.forEach((profile: any) => {
          const { userID, nick } = profile;
          const showName = nick || userID;
          this.userShowNameMap.set(userID, showName);
        });
      }).catch(() => {
        // 请求出错时捕获异常，避免阻塞代码执行流程
      });
    }
  }

  private substringByLength(str: string, len = 12) {
    return str.length > len ? `${str.slice(0, len)}...` : str;
  }

  private isOldUIKit(target: string) {
    const index = target.lastIndexOf('.');
    const flag = target.slice(0, index + 1);
    return this.t(target)?.startsWith(flag);
  }
}
