import type { Message } from '@tencentcloud/chat-react-native';
import { JSONToObject } from '../utils/common-utils';
import type { SendMessageParams } from '../type';
import { StoreName, typingConfig } from '../const';
import type { ITUIChatService } from '../interface/service';

export default class TypingHandler {
  private typingTo: string;
  private timer: number | null;
  private TUIChatService: ITUIChatService;
  constructor(TUIChatService: ITUIChatService) {
    this.TUIChatService = TUIChatService;
    this.typingTo = '';
    this.timer = null;
  }

  private getEngine() {
    return this.TUIChatService.getEngine();
  }

  /**
   * 清除 typing 状态
   */
  public clearTypingStatus() {
    if (this.getEngine().TUIStore.getData(StoreName.CHAT, 'typingStatus')) {
      this.getEngine().TUIStore.update(StoreName.CHAT, 'typingStatus', false);
    }
    this.timer && clearTimeout(this.timer);
    this.timer = null;
  }

  /**
   * 处理最后一条消息
   * @param { Message } lastMessage 最后一条消息
   */
  private handleLastMessage(lastMessage: Message) {
    const getEngine = this.getEngine();
    const currentTypingStatus = getEngine.TUIStore.getData(StoreName.CHAT, 'typingStatus');
    if (lastMessage.type === this.getEngine().TYPES.MSG_CUSTOM && lastMessage.flow === 'in') {
      const typingMessage = JSONToObject(lastMessage.payload.data);
      if (typingMessage.businessID === typingConfig.BUSINESS_ID) {
        const { typingStatus, userAction } = typingMessage;
        if (typingStatus === typingConfig.STATUS_START || userAction === typingConfig.ACTION_START_ID) {
          if (currentTypingStatus) {
            this.timer && clearTimeout(this.timer);
            this.timer = null;
          } else {
            getEngine.TUIStore.update(StoreName.CHAT, 'typingStatus', true);
          }
          this.timer = setTimeout(() => {
            this.clearTypingStatus();
          }, 30 * 1000);
          return getEngine.TUIStore.getData(StoreName.CHAT, 'typingStatus');
        }
        if (typingStatus === typingConfig.STATUS_END || userAction === typingConfig.ACTION_END_ID) {
          this.clearTypingStatus();
        }
      }
    }
    return undefined;
  }

  /**
   * 用于过滤 messageList
   * @param messageList 要进行更新的 messageList
   */
  public filterTypingMessage(messageList: Message[] = []) {
    if (messageList.length === 0) {
      return [];
    }
    const enableTyping = this.getEngine().TUIStore.getData(StoreName.APP, 'enableTyping');
    // 开启了正在输入功能时才处理从其他端收到的正输入消息
    if (enableTyping) {
      this.handleLastMessage(messageList[messageList.length - 1]);
    }
    return messageList.filter((item: Message) => {
      if (item.type === this.getEngine().TYPES.MSG_CUSTOM) {
        const typingMessage = JSONToObject(item.payload.data);
        if (typingMessage.businessID !== typingConfig.BUSINESS_ID) {
          return true;
        }
        return false;
      }
      return true;
    });
  }

  /**
   * 私有方法, 创建 typing 消息体
   * @param { number } status 1: 正在输入 0: 输入完成, 默认: 0
   * @param { string } to 接收方 ID
   */
  private createTypingMessage(status: number = typingConfig.STATUS_END, to = '') {
    const { BUSINESS_ID, STATUS_START, VERSION, ACTION_START_ID, ACTION_END_ID, ACTION_START, ACTION_END, NEED_TYPING } = typingConfig;
    const options: SendMessageParams = {
      to,
      conversationType: this.getEngine().TYPES.CONV_C2C,
      payload: {
        data: JSON.stringify({
          businessID: BUSINESS_ID,
          typingStatus: status,
          version: VERSION,
          userAction: status === STATUS_START ? ACTION_START_ID : ACTION_END_ID,
          actionParam: status === STATUS_START ? ACTION_START : ACTION_END,
        }),
        description: '',
        extension: '',
      },
      cloudCustomData: JSON.stringify({
        messageFeature: {
          needTyping: NEED_TYPING,
          version: VERSION,
        },
      }),
    };
    return options;
  }

  /**
   * 发送 typing 消息
   * @param { boolean } status true: 正在输入 false: 输入完成 默认: false
   * @param { string } to 接收方 ID
   */
  public sendTyping(status = false, to = '') {
    if (status) {
      this.typingTo = to;
      const typingMessage = this.createTypingMessage(typingConfig.STATUS_START, to);
      this.getEngine().TUIChat.sendCustomMessage(typingMessage, { onlineUserOnly: true });
    } else {
      this.sendTypingEnd();
    }
  }

  /**
   * 发送 typing 结束消息
   */
  public sendTypingEnd() {
    if (this.typingTo) {
      const typingMessage = this.createTypingMessage(typingConfig.STATUS_END, this.typingTo);
      this.getEngine().TUIChat.sendCustomMessage(typingMessage, { onlineUserOnly: true });
    }
    this.typingTo = '';
  }
}
