import type { ChatSDK } from '@tencentcloud/chat-react-native';
import TencentCloudChat from '@tencentcloud/chat-react-native';
// import TIMUploadPlugin from 'tim-upload-plugin';
import TIMProfanityFilterPlugin from 'tim-profanity-filter-plugin';
import EventCenter from './event-center';
import type { ITUIChatEngine, IEventCenter } from '../interface/engine';
import type {
  ITUITranslateService,
  ITUIConversationService,
  ITUIChatService,
  ITUIGroupService,
  ITUIUserService,
  ITUIFriendService,
  ITUIReportService,
} from '../interface/service';
import type { ITUIStore } from '../interface/store';
import type { LoginParams } from '../type';
import type { MountedList } from '../const';
import { StoreName } from '../const';
import { isUndefined } from '../utils/common-utils';
import { commercialAbility } from '../config';
import { IN_UNI_NATIVE_APP, IN_WX_MINI_APP } from '../utils/env';
import TUIGlobal from '../TUIGlobal/tui-global';
import OPTIONAL_MODULE from '../const/optional-module';

export default class ChatEngine implements ITUIChatEngine {
  static instance: ChatEngine;
  public isInited: boolean; // 是否执行了初始化
  public chat!: ChatSDK;
  public EVENT: any;
  public TYPES: any;
  public eventCenter!: IEventCenter;
  public TUIStore!: ITUIStore;
  public TUITranslate!: ITUITranslateService;
  public TUIConversation!: ITUIConversationService;
  public TUIChat!: ITUIChatService;
  public TUIGroup!: ITUIGroupService;
  public TUIUser!: ITUIUserService;
  public TUIFriend!: ITUIFriendService;
  public TUIReport!: ITUIReportService;
  private loginStatusPromise: Map<string, unknown>; // 保存 login Promise
  private userID: string; // 当前用户 ID
  private SDKAppID: number; // SDKAppID
  private userSig: string; // 用户签名

  constructor() {
    this.EVENT = TencentCloudChat.EVENT;
    this.TYPES = TencentCloudChat.TYPES;
    this.loginStatusPromise = new Map();
    this.userID = '';
    this.SDKAppID = 0;
    this.userSig = '';
    this.isInited = false;
  }

  /**
   * 获取 ChatEngine 实例
  */
  static getInstance() {
    if (!ChatEngine.instance) {
      ChatEngine.instance = new ChatEngine();
    }
    return ChatEngine.instance;
  }

  mount(name: MountedList, instance: any) {
    this[name] = instance;
  }

  async login(options: LoginParams) {
    const { chat, SDKAppID, userID, userSig } = options;
    const isOfficial = SDKAppID === 1400187352 || SDKAppID === 1400188366;
    await this.createChat(options);
    this.userID = userID;
    this.SDKAppID = SDKAppID;
    this.userSig =userSig;
    TUIGlobal.getInstance().initOfficial(isOfficial); // 兼容低版本 uikit
    this.TUIStore.update(StoreName.APP, 'isOfficial', isOfficial);
    this.TUIStore.update(StoreName.APP, 'SDKVersion', (TencentCloudChat as any).VERSION);
    this.eventCenter = new EventCenter(this);
    this.eventCenter.removeEvents(); // 移除注册的事件监听，避免重复注册
    this.resetStore();
    this.initService();
    if (chat && chat.isReady()) {
      console.log('TUIChatEngine.login ok, from TUICore.');
      this.TUIUser.getUserProfile();
      this.checkCommercialAbility();
      return Promise.resolve({});
    }
    this.registerPlugin(options);
    this.eventCenter.addEvent(this.EVENT.SDK_READY, () => {
      this.onSDKReady();
    });
    this.eventCenter.addEvent(this.EVENT.SDK_NOT_READY, () => {
      this.onSDKNotReady();
    });
    return this.loginChat(options);
  }

  logout() {
    this.userID = '';
    this.isInited = false;
    this.resetStore();
    return this.chat.logout();
  }

  isReady() {
    return this.chat?.isReady() || false;
  }

  setLogLevel(level: number) {
    if (!this.chat) {
      console.warn('TUIChatEngine 初始化未完成，请确认 TUIChatEngine.login 接口调用是否正常。');
      return;
    }
    this.chat.setLogLevel(level);
  }

  destroy() {
    // 解绑 IM 事件
    this.eventCenter.unbindIMEvents();
    this.isInited = false;
    this.resetStore();
    return this.chat.destroy();
  }

  getMyUserID() {
    return this.userID;
  }

  getContext() {
       return {
           chat: this.chat,
           userID: this.userID,
           SDKAppID: this.SDKAppID,
           userSig: this.userSig
       }
  }

  /**
   * 重置 Store
  */
  private resetStore() {
    this.TUIStore.reset(StoreName.CHAT);
    this.TUIStore.reset(StoreName.CONV);
    this.TUIStore.reset(StoreName.GRP);
    this.TUIStore.reset(StoreName.USER);
    this.TUIStore.reset(StoreName.SEARCH);
    this.TUIStore.reset(StoreName.FRIEND);
    this.TUIStore.reset(StoreName.CUSTOM);
    console.log('TUIChatEngine.resetStore ok.');
  }

  /**
   * 初始化 Service
  */
  private initService() {
    this.TUIChat.init();
    this.TUIConversation.init();
    this.TUIUser.init();
    this.initOptionalServices();
    this.isInited = true;
    console.log('TUIChatEngine.initService ok.');
  }

  private async initOptionalServices() {
    const version = (TencentCloudChat as any).VERSION;
    const arr = version.split('.');
    // Chat SDK 版本大于 3.3.0 支持
    if (arr[0] > 3 || (arr[0] === 3 && arr[1] >= 3 && arr[2] > 0)) {
      if (await this.chat.callExperimentalAPI('canIUseModule', [OPTIONAL_MODULE.GRP]) === true) {
        this.TUIGroup.init();
      }
      if (await this.chat.callExperimentalAPI('canIUseModule', [OPTIONAL_MODULE.SNS]) === true) {
        this.TUIFriend.init();
      }
    } else {
      this.TUIGroup.init();
      this.TUIFriend.init();
    }
  }

  private async createChat(options: LoginParams) {
    const { chat, ...rest } = options;
    if (this.chat != undefined) {
        return;
      }
    if (!isUndefined(chat)) {
      this.chat = chat;
      return;
    }
    this.chat = await TencentCloudChat.create({
      ...rest,
      scene: 'chat-uikit-engine',
    });
  }

  private loginChat(options: LoginParams) {
    const { userID, userSig } = options;
    return new Promise((resolve, reject) => {
      this.chat.login({ userID, userSig }).then((imResponse: any) => {
        this.checkCommercialAbility();
        if (imResponse.data.repeatLogin && this.chat.isReady()) {
          resolve(imResponse);
        }
        this.loginStatusPromise.set('login', { resolve, reject, imResponse });
      }).catch((error: any) => {
        reject(error);
      });
    });
  }

  /**
   * 注册插件，用于文件上传、本地审核、离线推送插件
  */
  private registerPlugin(options: LoginParams) {
    // const {
    //   useUploadPlugin = false,
    //   useProfanityFilterPlugin = false,
    //   TIMPush = undefined,
    //   pushConfig = undefined,
    //   TUIOfflinePush = undefined,
    //   offlinePushConfig = undefined,
    // } = options;
    // if (useUploadPlugin === true) {
    //   this.chat.registerPlugin({ 'tim-upload-plugin': TIMUploadPlugin });
    // }
    // if (useProfanityFilterPlugin === true) {
    //   this.chat.registerPlugin({ 'tim-profanity-filter-plugin': TIMProfanityFilterPlugin });
    // }
    // // uniApp 打包小程序时 IN_UNI_NATIVE_APP 为 true，所以此处需要增加条件
    // if (IN_UNI_NATIVE_APP && !IN_WX_MINI_APP && !isUndefined(TIMPush) && !isUndefined(pushConfig)) {
    //   this.chat.registerPlugin({ 'tim-push': TIMPush, pushConfig });
    // }
    // // uniApp 打包小程序时 IN_UNI_NATIVE_APP 为 true，所以此处需要增加条件
    // if (IN_UNI_NATIVE_APP && !IN_WX_MINI_APP && !isUndefined(TUIOfflinePush) && !isUndefined(offlinePushConfig)) {
    //   this.chat.registerPlugin({ 'tim-offline-push-plugin': TUIOfflinePush, offlinePushConfig });
    // }
  }

  private onSDKReady() {
    if (this.loginStatusPromise.has('login')) {
      const handler: any = this.loginStatusPromise.get('login');
      handler.resolve(handler.imResponse);
      // 独立使用 engine 登录成功 SDK Ready 后获取一次自己的资料
      this.TUIUser.getUserProfile();
    }
    this.loginStatusPromise.delete('login');
  }

  private onSDKNotReady() {
    if (this.loginStatusPromise.has('login')) {
      const handler: any = this.loginStatusPromise.get('login');
      handler.reject(new Error('sdk not ready'));
    }
    this.loginStatusPromise.delete('login');
    this.resetStore();
  }

  // 登录成功后校验商业化能力位
  private checkCommercialAbility() {
    Object.keys(commercialAbility).forEach((key: string) => {
      const ability: number = commercialAbility[key];
      this.chat.callExperimentalAPI('isCommercialAbilityEnabled', ability).then((imResponse: any) => {
        const { enabled = false } = imResponse.data;
        this.TUIStore.update(StoreName.APP, key, enabled);
      });
    });
  }
}
