import i18next from 'i18next';
import type { ITUITranslateService } from '../interface/service';
import TUIBase from '../tui-base';
import { isUndefined } from '../utils/common-utils';
import { IN_WX_MINI_APP } from '../utils/env';

export default class TUITranslateService extends TUIBase implements ITUITranslateService {
  static instance: TUITranslateService;
  private languages: any;
  private keyMap: Record<string, string>;

  constructor() {
    super();
    this.languages = null;
    this.keyMap = {};
  }

  /**
   * 获取 TUITranslateService 实例
  */
  static getInstance() {
    if (!TUITranslateService.instance) {
      TUITranslateService.instance = new TUITranslateService();
    }
    return TUITranslateService.instance;
  }

  public provideLanguages(languages: Record<string, any>) {
    if (IN_WX_MINI_APP) {
      return;
    }
    this.languages = languages;
    // Set key map if languages['zh-CN'] has `Chat`.
    if (languages['zh-CN']?.Chat) {
      this.setKeyMap(languages['zh-CN']);
    }
    console.log('TUITranslateService.provideLanguages ok.');
  }

  public useI18n(lng?: string) {
    if (IN_WX_MINI_APP) {
      return;
    }
    if (!this.languages) {
      console.warn('TUITranslateService.useI18next not have messages.');
      return;
    }

    const resources: { [key: string]: { translation: any } } = {};
    for (const [key, language] of Object.entries(this.languages)) {
      const resourceKey = key === 'zh_cn' ? 'zh' : key;
      resources[resourceKey] = { translation: language };
    }

    (i18next as any).init({
      compatibilityJSON: 'v3', // 打包 app 时, compatibilityJSON 版本不兼容，需要调整到 v3,
      lng: lng || 'zh', // target language
      fallbackLng: 'zh', // default language
      detection: {
        // order and from where user language should be detected
        order: ['querystring', 'navigator'],
        // cache user language on
        caches: ['localStorage', 'cookie'],
      },
      resources,
    });

    // report i18n feature
    let _langReportKey = this.normalizeLanguageKey(lng || 'zh');
    this.getEngine().TUIReport?.reportFeature(201, _langReportKey);
  }

  public changeLanguage(language: string) {
    // report i18n feature
    let _langReportKey = this.normalizeLanguageKey(language);
    this.getEngine().TUIReport.reportFeature(201, _langReportKey);

    // change language
    return (i18next as any).changeLanguage(language);
  }

  private normalizeLanguageKey(language: string): string {
    if (language.includes('en')) return 'en-us';
    if (language.includes('tw')) return 'zh-tw';
    if (language.includes('zh') || language.includes('cn')) return 'zh-cn';
    if (language.includes('ko') || language.includes('kr')) return 'ko-kr';
    if (language.includes('ja') || language.includes('jp')) return 'ja-jp';
    return 'other-' + language;
  }

  public t(key: string) {
    // case1: 小程序不需要支持国际化翻译
    // case2: 直接集成 ChatEngine 未提供语言包时需要返回默认解析值
    if (!this.languages) {
      let ret: string = key;
      if (!isUndefined(key) && key.indexOf('.') > -1) {
        ret = key.split('.').pop() || '';
      }
      return ret;
    }
    const _key = this.keyMap[key];
    if (_key) {
      return (i18next as any).t(_key);
    }
    return (i18next as any).t(key);
  }

  // example: Chat.CUSTOM_MESSAGE -> TUIChat.[自定义消息]
  private setKeyMap(target: Record<string, any>, prefix?: string) {
    for (const key in target) {
      if (Object.prototype.hasOwnProperty.call(target, key)) {
        const newPrefix = prefix ? `${prefix}.${key}` : key;
        if (target[key] && typeof target[key] === 'object' && !Array.isArray(target[key]) && target[key] !== null) {
          this.setKeyMap(target[key], newPrefix);
        } else {
          let newKey = prefix ? `${prefix}.${target[key]}` : target[key];
          newKey = this.replaceKey(newKey);
          this.keyMap[newKey] = `${prefix}.${key}`;
        }
      }
    }
  }

  // example: Chat.[自定义消息] -> TUIChat.[自定义消息]
  private replaceKey(key: string) {
    if (key.startsWith('Chat')) {
      return key.replace('Chat', 'TUIChat');
    }
    if (key.startsWith('Conversation')) {
      return key.replace('Conversation', 'TUIConversation');
    }
    if (key.startsWith('Message')) {
      return key.replace('Message', 'message.tip');
    }
    if (key.startsWith('Call')) {
      return key.replace('Call', 'message.custom');
    }
    return key;
  }
}
