import TUIBase from '../tui-base';
import type { ITUIUserService } from '../interface/service';
import type { SwitchUserStatusParams, UpdateMyProfileParams, UserIDListParams } from '../type';
import UserProfileHandler from './user-profile-handler';
import UserStatusHandler from './user-status-handler';
import { StoreName } from '../const';
import { getStorageItem, setStorageItem } from '../utils/storage';

export default class TUIUserService extends TUIBase implements ITUIUserService {
  static instance: TUIUserService;
  private userProfileHandler: UserProfileHandler;
  private userStatusHandler: UserStatusHandler;
  constructor() {
    super();
    this.userProfileHandler = new UserProfileHandler(this);
    this.userStatusHandler = new UserStatusHandler(this);
  }

  /**
   * 获取 TUIUserService 实例
  */
  static getInstance() {
    if (!TUIUserService.instance) {
      TUIUserService.instance = new TUIUserService();
    }
    return TUIUserService.instance;
  }

  init() {
    const chatEngine = this.getEngine();
    // 监听用户被踢通知
    chatEngine.eventCenter.addEvent(chatEngine.EVENT.KICKED_OUT, this.onKickedOut.bind(this));
    // 监听用户网络变更通知
    chatEngine.eventCenter.addEvent(chatEngine.EVENT.NET_STATE_CHANGE, this.onNetStateChange.bind(this));

    this.userProfileHandler.init();
    this.userStatusHandler.init();
    this.initMessageReadReceipt();
  }

  private onKickedOut(data: any) {
    this.getEngine().TUIStore.update(StoreName.USER, 'kickedOut', data.type);
  }

  private onNetStateChange(data: any) {
    this.getEngine().TUIStore.update(StoreName.USER, 'netStateChange', data.state);
  }

  // 初始化消息阅读状态开关
  private initMessageReadReceipt() {
    // Service 初始化时从 localStorage 中获取上一次设置的消息阅读状态记录，更新到 Store
    // UI 层可通过 TUIStore.getData('displayMessageReadReceipt') 获取到初始值
    const displayMessageReadReceipt = getStorageItem('displayMessageReadReceipt') || true;
    this.getEngine().TUIStore.update(StoreName.USER, 'displayMessageReadReceipt', displayMessageReadReceipt);
  }

  public switchUserStatus(options: SwitchUserStatusParams) {
    return this.userStatusHandler.switchUserStatus(options);
  }

  public switchMessageReadStatus(displayMessageReadReceipt: boolean) {
    this.getEngine().TUIStore.update(StoreName.USER, 'displayMessageReadReceipt', displayMessageReadReceipt);
    setStorageItem('displayMessageReadReceipt', displayMessageReadReceipt);
  }

  // 用户资料相关的方法
  public getUserProfile(userIDList?: UserIDListParams) {
    return this.userProfileHandler.getUserProfile(userIDList);
  }

  public updateMyProfile(options: UpdateMyProfileParams) {
    return this.userProfileHandler.updateMyProfile(options);
  }

  public addToBlacklist(options: UserIDListParams) {
    return this.userProfileHandler.addToBlacklist(options);
  }

  public removeFromBlacklist(options: UserIDListParams) {
    return this.userProfileHandler.removeFromBlacklist(options);
  }

  // 用户状态相关的方法
  public getUserStatus(options: UserIDListParams) {
    return this.userStatusHandler.getUserStatus(options);
  }

  public subscribeUserStatus(options: UserIDListParams) {
    return this.userStatusHandler.subscribeUserStatus(options);
  }

  public unsubscribeUserStatus(options?: UserIDListParams) {
    return this.userStatusHandler.unsubscribeUserStatus(options);
  }
}
