import type { SwitchUserStatusParams, UserIDListParams } from '../type';
import { StoreName, UserStatus } from '../const';
import { setStorageItem, getStorageItem } from '../utils/storage';
import type { ITUIUserService } from '../interface/service';

export default class UserStatusHandler {
  private TUIUserService: ITUIUserService;
  constructor(TUIUserService: ITUIUserService) {
    this.TUIUserService = TUIUserService;
  }

  private getEngine() {
    return this.TUIUserService.getEngine();
  }

  init() {
    const chatEngine = this.getEngine();
    // Service 初始化时从 localStorage 中获取上一次设置用户状态记录，更新到 Store
    // UI 层可通过 TUIStore.getData('displayOnlineStatus') 获取到初始值
    const displayOnlineStatus = getStorageItem('displayOnlineStatus') || false;
    chatEngine.TUIStore.update(StoreName.USER, 'displayOnlineStatus', displayOnlineStatus);

    // 开启状态显示后默认订阅所有 C2C 会话用户状态
    chatEngine.eventCenter.addEvent(chatEngine.EVENT.USER_STATUS_UPDATED, this.onUserStatusUpdated.bind(this));
    this.onConversationListUpdated();
  }

  private onConversationListUpdated() {
    const chatEngine = this.getEngine();
    chatEngine.TUIStore.watch(StoreName.CONV, {
      conversationList: () => {
        const enabledOnlineStatus = chatEngine.TUIStore.getData(StoreName.APP, 'enabledOnlineStatus');
        const displayOnlineStatus = chatEngine.TUIStore.getData(StoreName.USER, 'displayOnlineStatus');
        // 购买了商业化能力且用户打开了用户状态显示，才需要启动订阅流程
        if (enabledOnlineStatus && displayOnlineStatus) {
          this.startToSubscribe();
        }
      },
    });
  }

  private onUserStatusUpdated(userStatusList: any[]) {
    this.getEngine().TUIStore.update(StoreName.USER, 'userStatusList', userStatusList);
  }

  public switchUserStatus(options: SwitchUserStatusParams) {
    const chatEngine = this.getEngine();
    const { displayOnlineStatus = false } = options;

    chatEngine.TUIStore.update(StoreName.USER, 'displayOnlineStatus', displayOnlineStatus);
    setStorageItem('displayOnlineStatus', displayOnlineStatus);

    const enabledOnlineStatus = chatEngine.TUIStore.getData(StoreName.APP, 'enabledOnlineStatus');
    // 没有购买商业化能力时，调用 switchUserStatus 只会更新缓存，不需要发起订阅和取消订阅操作
    if (!enabledOnlineStatus) {
      console.warn(`UserStatusHandler.switchUserStatus enabledOnlineStatus:${enabledOnlineStatus} displayOnlineStatus:${displayOnlineStatus}`);
      return;
    }
    if (displayOnlineStatus === true) {
      this.startToSubscribe();
    } else {
      this.unsubscribeUserStatus();
    }
  }

  public getUserStatus(options: UserIDListParams) {
    const chatEngine = this.getEngine();
    return chatEngine.chat.getUserStatus(options).then((imResponse: any) => {
      const { successUserList } = imResponse.data;
      if (successUserList.length > 0) {
        chatEngine.TUIStore.update(StoreName.USER, 'userStatusList', successUserList);
      }
      return imResponse;
    }).catch((error: any) => Promise.reject(error));
  }

  public subscribeUserStatus(options: UserIDListParams) {
    const len = options?.userIDList?.length || 0;
    if (len === 0) {
      console.warn(`UserStatusHandler.subscribeUserStatus userID count:${len}`);
      return Promise.resolve();
    }
    return this.getEngine().chat.subscribeUserStatus(options);
  }

  public unsubscribeUserStatus(options?: UserIDListParams) {
    const chatEngine = this.getEngine();
    // 不传参数或传入的 userIDList 为空表示取消所有订阅
    const { userIDList = [] } = options || {};
    return chatEngine.chat.unsubscribeUserStatus({ userIDList }).then((imResponse: any) => {
      const { failureUserList } = imResponse.data;
      const unsubscribedList: any[] = [];
      userIDList.forEach((userID: string) => {
        if (!failureUserList.includes(userID)) {
          unsubscribedList.push({
            userID,
            statusType: UserStatus.UNSUB_USER,
          });
        }
      });
      chatEngine.TUIStore.update(StoreName.USER, 'userStatusList', unsubscribedList);
      return imResponse;
    }).catch((error: any) => Promise.reject(error));
  }

  /**
   * 启动用户状态订阅
   */
  private startToSubscribe() {
    const chatEngine = this.getEngine();
    const conversationList = chatEngine.TUIStore.getData(StoreName.CONV, 'conversationList');
    const currentUserStatusList = chatEngine.TUIStore.getData(StoreName.USER, 'userStatusList');
    const addedUserStatusList: any[] = [];
    // 从会话列表中过滤出新增的 C2C 会话 userID
    conversationList.forEach((conversation: any) => {
      if (conversation.type === chatEngine.TYPES.CONV_C2C) {
        const userID = conversation.conversationID.replace(chatEngine.TYPES.CONV_C2C, '');
        if (!currentUserStatusList.has(userID)) {
          addedUserStatusList.push(userID);
        }
      }
    });
    if (addedUserStatusList.length === 0) {
      return;
    }
    if (chatEngine.chat.isReady()) {
      // 新创建的 C2C 会话需要自动订阅和主动拉取一次用户状态，获取用户状态成功会更新 userStatusList
      this.subscribeUserStatus({ userIDList: addedUserStatusList });
      this.getUserStatus({ userIDList: addedUserStatusList });
    }
  }
}
