import type { PinConversationParams, MuteConversationParams, SetConversationDraftParams } from '../../type';

/**
 * @interface TUIConversationService
*/
export interface ITUIConversationService {

  /**
   * 初始化 Service
   * @private
   */
  init(): void;

  /**
   * 切换会话
   * @function
   * @param {String} conversationID 会话 ID，conversationID 生成规则：单聊（`C2C${userID}`），群聊（`GROUP${groupID}`）。
   * @example
   * // 切换到 public001 群会话
   * let promise = TUIConversationService.switchConversation('GROUPpublic001');
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  switchConversation(conversationID: string): Promise<any>;

  /**
   * 获取会话列表
   * @function
   * @private
   */
  getConversationList(): Promise<any>;

  /**
   * 获取会话资料
   * 注意：Service API 主要用于跨组件时调用
   * @function
   * @param {String} conversationID 会话 ID
   * @example
   * // 获取 public001 群会话资料
   * let promise = TUIConversationService.getConversationProfile('GROUPpublic001');
   * promise.then((chatResponse) => {
   *  console.log(chatResponse.data.conversation); // 会话资料
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  getConversationProfile(conversationID: string): Promise<any>;

  /**
   * 删除会话
   * @function
   * @param {String} conversationID 会话 ID
   * @private
   */
  deleteConversation(conversationID: string): Promise<any>;

  /**
   * 置顶会话
   * @function
   * @param {PinConversationParams} options 参数选项
   * @private
   */
  pinConversation(options: PinConversationParams): Promise<any>;

  /**
   * 设置消息免打扰
   * @function
   * @param {MuteConversationParams} options 参数选项
   * @private
   */
  muteConversation(options: MuteConversationParams): Promise<any>;

  /**
   * 一键清空历史消息
   * @function
   * @param {String} conversationID 会话 ID
   * @example
   * let promise = TUIConversationService.clearHistoryMessage('GROUPpublic001');
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  clearHistoryMessage(conversationID: string): Promise<any>;

  /**
   * 设置会话草稿
   * @function
   * @param {SetConversationDraftParams} options 参数选项
   * @example
   * // 设置会话草稿
   * let promise = TUIConversationService.setConversationDraft({
   *   conversationID: 'GROUPpublic001',
   *   darftInfo: {
   *    html: 'test';
   *    abstract: 'test';
   *   }
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   * @example
   * // 清空会话草稿
   * let promise = TUIConversationService.setConversationDraft({
   *   conversationID: 'GROUPpublic001',
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  setConversationDraft(options: SetConversationDraftParams): Promise<any>;
}
