import type TUIBase from '../../tui-base';
import type { SwitchUserStatusParams, UserIDListParams, UpdateMyProfileParams } from '../../type';

/**
 * @interface TUIUserService
*/
export interface ITUIUserService extends TUIBase {

  /**
   * 初始化 Service
   * @function
   * @private
   */
  init(): void;

  /**
   * 用户状态切换开关
   * @function
   * @param {SwitchUserStatusParams} options 用户状态控制参数
   * @example
   * // 开启用户在线状态显示
   * TUIUserService.switchUserStatus({ displayOnlineStatus: true });
   * @example
   * // 关闭用户在线状态显示
   * TUIUserService.switchUserStatus({ displayOnlineStatus: false });
   */
  switchUserStatus(options: SwitchUserStatusParams): void;

  /**
   * 消息阅读状态切换开关
   * @function
   * @param {boolean} displayMessageReadReceipt 消息阅读状态控制参数
   * @example
   * // 开启用户消息阅读状态
   * TUIUserService.switchMessageReadStatus(true);
   * @example
   * // 关闭用户消息阅读状态
   * TUIUserService.switchMessageReadStatus(false);
   */
  switchMessageReadStatus(displayMessageReadReceipt: boolean): void;

  /**
   * 获取用户资料
   * @function
   * @param {UserIDListParams} options [options = undefined] 用户 ID 列表，不传 options 默认查询自己的资料
   * @example
   * // 查询自己的资料
   * let promise = TUIUserService.getUserProfile();
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   * @example
   * // 查询其他用户的资料
   * let promise = TUIUserService.getUserProfile({ userIDList: ['user1', 'user2'] });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  getUserProfile(options?: UserIDListParams): Promise<any>;

  /**
   * 更新自己的资料信息
   * @function
   * @param {UpdateMyProfileParams} options 参数选项
   * @example
   * // 更新自己的昵称
   * let promise = TUIUserService.updateMyProfile({
   *  nick: 'newNick'
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  updateMyProfile(options: UpdateMyProfileParams): Promise<any>;

  /**
   * 添加用户到黑名单列表。将用户加入黑名单后可以屏蔽来自 TA 的所有消息，因此该接口可以实现“屏蔽该用户消息”的功能。
   * @function
   * @param {UserIDListParams} options 参数选项
   * @example
   * let promise = TUIUserService.addToBlacklist({ userIDList: ['user1', 'user2'] });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  addToBlacklist(options: UserIDListParams): Promise<any>;

  /**
   * 从黑名单列表中移除
   * @function
   * @param {UserIDListParams} options 参数选项
   * @example
   * let promise = TUIUserService.removeFromBlacklist({ userIDList: ['user1', 'user2'] });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  removeFromBlacklist(options: UserIDListParams): Promise<any>;

  /**
   * 获取用户状态
   * @function
   * @param {UserIDListParams} options 参数选项
   * @private
   * @example
   *  // 查询 user1、user2 的在线状态
   * let promise = TUIUserService.getUserStatus({ userIDList: ['user1', 'user2'] });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  getUserStatus(options: UserIDListParams): Promise<any>;

  /**
   * 订阅用户状态
   * @function
   * @param {UserIDListParams} options 参数选项
   * @private
   * @example
   * // 订阅 user1、user2 的在线状态
   * let promise = TUIUserService.subscribeUserStatus({ userIDList: ['user1', 'user2'] });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  subscribeUserStatus(options: UserIDListParams): Promise<any> | void;

  /**
   * 取消订阅用户状态
   * @function
   * @param {UserIDListParams} [options] 用户 userID 列表，不传 options 时，取消当前所有订阅
   * @private
   * @example
   * // 取消 user1、user2 在线状态的订阅
   * let promise = TUIUserService.unsubscribeUserStatus({ userIDList: ['user1', 'user2'] });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   * @example
   * // 取消所有用户在线状态的订阅
   * let promise = TUIUserService.unsubscribeUserStatus();
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  unsubscribeUserStatus(options?: UserIDListParams): Promise<any>;
}
